/*
  buffer capabilites for a stream
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */

#include <producer/yaf/yafStream.h>

// Counter is used for debugging reasons (to distinguish
// between the threads. And to create an unique (hopefully) fifo name

static int counter=0;



static void *readloopThread(void *arg){
  ((YafStream*)arg)->readloop();
  return NULL;
}   



YafStream::YafStream(YafProducer* producer) {
  int rNum;

  this->producer=producer;
  pthread_mutex_init(&mut,NULL);
  pthread_cond_init(&cond,NULL);

  Buffer randomName(60);
  streamCounter=counter;
  randomName.append("/tmp/.kmpg.");

  rNum=time(NULL);
  randomName.append(rNum);
  randomName.append(counter);
  randomName.append(getuid());
  randomName.append(getpid());
  counter++;
  

  fifoCommand=new Buffer(50);
  fifoCommand->clear();
  fifoCommand->append("outputfile ");
  fifoCommand->append(randomName.getData());
  fifoCommand->append("\n");
  
  size=1024*1024*1;
  isShutdown=true;

  notifyBridge=new NotifyMainThread(this);


  fifo=new Fifo(randomName.getData(),O_RDONLY);
  ringBufferNT=new RingBufferNT(size,16384);
  bufferUnderrun=__BUFFER_UNDERRUN_DETECTED;
  buffUnderRunSize=size/4;
  buffOverRunSize=size/3;
  start();
}


YafStream::~YafStream() {
  stop();
  delete ringBufferNT;
  delete fifoCommand;
  delete fifo;
  delete notifyBridge;
}




void YafStream::start() {
  if (isShutdown == true) {
    isShutdown=false;
    pthread_create(&tr,NULL,readloopThread,this);
    return;
  }
  pthread_mutex_unlock(&mut);
}



void YafStream::stop() {
  void* ret;
  if (isShutdown == false) {
    isShutdown=true;
    fifo->close();
    clear();
    ringBufferNT->exitWaitForSpace();
    pthread_join(tr,&ret);
    return;
  }
}



/**
   These Functions are entered by the "reader" thread [START]

   Note: The Streambuffer is _not_ filled if we have a buffer
   we simply set the StartPtr to the memory of the buffer.
   If we have  not buffer we set the ptr to the read stream.

*/



void YafStream::fillBuffer(AudioBuffer* buffer) {
  int pSize=buffer->getPreferredFillGrade();
  MemChunk* gmem;

  gmem=ringBufferNT->requestMemChunk(pSize);
  buffer->setMemChunk(gmem);

  // Note : in order to make the threads as parallel as possible
  // we have a single race condition which must be handled
  // seperatly. This race condition comes up, because
  // the reader thread does not wait on the waitForData
  // Mutex (as expected by the ringBufferNT, and which is
  // true in nearyl all cases) We must make sure that if
  // we (think) we have no data, we make another (this time
  // mutexed) try. to be _really_ sure. I think this
  // does not overcomplicate the situation. And the gain
  // that in all other cases the both threads run unmutexed
  // this is ok.

  if (gmem->getLen() == 0) {     // bufferunderrun or a race condition?
    pthread_mutex_lock(&mut);    // block writer thread

    gmem=ringBufferNT->requestMemChunk(pSize);
    buffer->setMemChunk(gmem);

    if (gmem->getLen() == 0) {
      cout << "*************** Bufer underrun **********"
	   << "request was :"<<pSize <<endl;
      ringBufferNT->exitWaitForSpace();
      bufferUnderrun=__BUFFER_UNDERRUN_DETECTED;
    } else {
      cout << "++++++++++++++++++++++++++++++++++++++++++++ backend thread cheated us !!!!!"<<endl;
    }
    pthread_mutex_unlock(&mut);
  }
  buffer->setReadBytes(ringBufferNT->getReadBytes());
  return;
}
  


/**
   These Functions are entered by the "reader" thread [END]
*/


void YafStream::processNotify(char msgId) {
  producer->bufferFilledNotify((int)msgId);
}


/**
   The readloop is the thread which reads the fifo and fills the
   buffer
*/

void YafStream::readloop() {
  int didRead;
  int fillgrade;
  MemChunk* gmem=new MemChunk(0);
  int skipCounter=0;
  int preferredSize=8192;

  pthread_mutex_lock(&mut);
  while(isShutdown==false) {
    if (fifo->isOpen() == false) {
      pthread_cond_wait(&cond,&mut);
      skipCounter=0;
      continue;
    }
    if (bufferUnderrun==__BUFFER_UNDERRUN_DETECTED) {
      notifyBridge->sendNotify((char)false);
      bufferUnderrun=__BUFFER_UNDERRUN_NOTIFIED;
    }
    if (bufferUnderrun==__BUFFER_UNDERRUN_NOTIFIED) {
      fillgrade=ringBufferNT->getFillgrade();
      if  (fillgrade>buffOverRunSize) {
	cout << "+++++++++++++ buffer well filled +++++++++++"<<endl;
	bufferUnderrun=__BUFFER_UNDERRUN_FALSE_DETECTED;
	pthread_mutex_unlock(&mut);
	notifyBridge->sendNotify((char)true);
	bufferUnderrun=__BUFFER_UNDERRUN_FALSE_NOTIFIED;
	pthread_mutex_lock(&mut);
	continue;  
      }
    }

    pthread_mutex_unlock(&mut);
    ringBufferNT->getWriteArea(gmem,preferredSize);
   
    if ( gmem->getLen() < preferredSize ){
      ringBufferNT->waitForSpace(preferredSize);
      ringBufferNT->getWriteArea(gmem,preferredSize);
    }
    didRead=fifo->read(gmem->getPtr(),gmem->getLen());
    if (didRead < 0) {
      cout << "Fifo error <= 0"<<endl;
      fifo->close();
      continue;
    }
    pthread_mutex_lock(&mut);
    if (lWriteToRing) {
      ringBufferNT->forwardWritePtr(didRead);
     } else {
       //cout << "Skip read bytes :"<<didRead<<endl;
     }
  }
  pthread_mutex_unlock(&mut);
  delete gmem;
}


/**
   These Functions are entered by the "reader" thread [END]
*/




  
/**

   
*/
char* YafStream::getFifoCommand() {
  return fifoCommand->getData();
}


void YafStream::lockReaderThread() {
  pthread_mutex_lock(&mut);
}


void YafStream::unlockReaderThread() {
  pthread_cond_signal(&cond);
  pthread_mutex_unlock(&mut);
}




void YafStream::openFifo() {
  fifo->open();
}
  
void YafStream::close() {
  setWriteRing(false);
  clear();
  //client->send("closeoutputfile\n");
}

  
void YafStream::setWriteRing(int lWrite) {
  pthread_mutex_lock(&mut);
  lWriteToRing=lWrite;
  pthread_mutex_unlock(&mut);
}
  

void YafStream::clear() {
  pthread_mutex_lock(&mut);
  bufferUnderrun=__BUFFER_UNDERRUN_DETECTED;
  ringBufferNT->emptyBuffer();
  
  pthread_mutex_unlock(&mut);
}



