//
//  kmpg
//
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <iostream.h>

#include <qmsgbox.h>

#include "kmpg.h"
#include <yafBridge/frontendDecoder.h>
#include "fileinfo.h"

KMpg::KMpg( QWidget *parent, const char *name ) :
   KTopLevelWidget( name )
{
   equalizer = new Equalizer();
   prefsdialog = new Preferences();
   playlistMgr = new PlaylistMgr();
   playlistDlg = new Playlist( playlistMgr );
   decoder = new FrontendDecoderSPlay( prefsdialog );
   filedialog = NULL;

   connect( decoder, SIGNAL(stopped(FrontendDecoder::stopReason)), SLOT(stopped(FrontendDecoder::stopReason)) );
   connect( equalizer, SIGNAL(sigChanged(int*, int*)), decoder, SLOT(equalizerChanged(int*, int*)) );
   connect( prefsdialog, SIGNAL(sigChanged()), decoder, SLOT(applyPreferences()) );

   connect( playlistMgr, SIGNAL(sigTrackSelected(KURL, bool)), SLOT(openUrl(KURL, bool)) );

   // DND init
   dropZone = new KDNDDropZone( this, DndURL );
   connect( dropZone, SIGNAL( dropAction( KDNDDropZone* )), SLOT( onDrop( KDNDDropZone*)));

   // Draw the panel
   setCaption( "MPEG Audio Player" );
   ledPanel = new LedPanel( this, playlistMgr, decoder );
   connect( ledPanel, SIGNAL(sizeChanged()), SLOT(ledPanelResized()) );

   setupMenubar();

   setView(ledPanel, FALSE);
   updateRects();

   // Session management support
   kapp->enableSessionManagement( FALSE );
   if( kapp->isRestored() )
   {
      cerr << "Restoring kmpg session" << endl;
      if( canBeRestored( 1 ) )
	 restore( 1 );
   }

   prefsdialog->slotChange();
   prefsdialog->applyPrefs();
   
   // Open file given as parameter
   if (qApp->argc()>=2)
      openUrl ( qApp->argv()[1], FALSE );
}

KMpg::~KMpg()
{
   //cerr << "KMpg::~KMpg(), deleting [";
   //delete menuBar; // this segfaults if the menubar is floated???
   delete decoder;
   delete playlistMgr;
}

void KMpg::saveProperties( KConfig* config )
{
   config->writeEntry("ShowTrackCal", view->isItemChecked(showTrackCal_id));
   config->writeEntry("ShowSlider", view->isItemChecked(showSlider_id));
   config->writeEntry("ShowControls", view->isItemChecked(showControls_id));

   playlistMgr->saveProperties( config );
   playlistDlg->saveProperties( config );
}

void KMpg::readProperties( KConfig* config )
{
   cerr << "reading Properties" << endl;
   if( !config->readNumEntry("ShowTrackCal", 1) ) {
      view->setItemChecked(showTrackCal_id, FALSE);
      showTrackCal(); } // Disables track calendar
   else
      view->setItemChecked(showTrackCal_id, TRUE);

   if( !config->readNumEntry("ShowSlider", 1) ) {
      view->setItemChecked(showSlider_id, FALSE);
      showSlider(); }
   else
      view->setItemChecked(showSlider_id, TRUE);

   if( !config->readNumEntry("ShowControls", 1) ) {
      view->setItemChecked(showControls_id, FALSE);
      showControls(); }
   else
      view->setItemChecked(showControls_id, TRUE);

   playlistMgr->readProperties( config );
   playlistDlg->readProperties( config );
}

void KMpg::setupMenubar()
{
   menuBar = new KMenuBar (this);

   file = new QPopupMenu;
   file->insertItem( klocale->translate("Open..."), this, 
		     SLOT(openClicked()), CTRL+Key_O );
   file->insertItem( klocale->translate("Properties"), ledPanel,
		     SLOT(viewProperties()) );
   file->insertSeparator();
   file->insertItem( klocale->translate("Quit"), kapp,
		     SLOT(quit()), ALT+Key_Q );

   view = new QPopupMenu;
   showMenubar_id = view->insertItem( klocale->translate("Show Menubar"), this,
				      SLOT(showMenubar()) );
   showTrackCal_id = view->insertItem( klocale->translate("Show Trackcalendar"),
				       this,
				       SLOT(showTrackCal()) );
   showSlider_id = view->insertItem( klocale->translate("Show Slider"), this, 
				     SLOT(showSlider()) );
   showControls_id = view->insertItem( klocale->translate("Show Controls"), this, 
				       SLOT(showControls()) );
   view->insertSeparator();
   view->insertItem( klocale->translate("Playlist Editor..."), playlistDlg, 
		     SLOT(slotShow()), ALT+Key_P );
   view->insertItem( klocale->translate("Equalizer..."), equalizer,
		     SLOT(slotShow()), ALT+Key_E );
   view->insertSeparator();
   view->insertItem( klocale->translate("Preferences..."), prefsdialog, 
		     SLOT(slotShow()) );
   view->setCheckable( TRUE );
   view->setItemChecked( showMenubar_id, TRUE );
   view->setItemChecked( showTrackCal_id, TRUE );
   view->setItemChecked( showSlider_id, TRUE );
   view->setItemChecked( showControls_id, TRUE );

   QString aboutStr = "kmpg-" VERSION "\n\n";
   aboutStr += "(c) 1997 Bjarni Ivarsson (Bjarni.Ivarsson@swipnet.se)\n\n";
   aboutStr += klocale->translate("An MPEG audio player for the KDE " \
				  "Desktop Environment\n" \
				  "See Help for more information");

   menuBar->insertItem( klocale->translate("&File"), file );
   menuBar->insertItem( klocale->translate("&View"), view );
   menuBar->insertSeparator();
   menuBar->insertItem( klocale->translate("&Help"),
			kapp->getHelpMenu(FALSE, aboutStr) );

   menuBar->show();
   setMenu(menuBar);
}

void KMpg::showMenubar()
{
   view->setItemChecked(showMenubar_id, 
			!view->isItemChecked(showMenubar_id));
   if( view->isItemChecked(showMenubar_id) )
      menuBar->show();
   else
      menuBar->hide();
   updateRects();
}

void KMpg::showTrackCal()
{
   view->setItemChecked(showTrackCal_id, 
			!view->isItemChecked(showTrackCal_id));
   ledPanel->showTrackCal( view->isItemChecked(showTrackCal_id) );
}

void KMpg::showSlider()
{
   view->setItemChecked(showSlider_id, 
			!view->isItemChecked(showSlider_id));
   ledPanel->showSlider( view->isItemChecked(showSlider_id) );
}

void KMpg::showControls()
{
   view->setItemChecked(showControls_id, 
			!view->isItemChecked(showControls_id));
   ledPanel->showControls( view->isItemChecked(showControls_id) );
}

void KMpg::closeEvent ( QCloseEvent *e ) 
{
   prefsdialog->applyPrefs();

//   playlistMgr->savePrefs();
   if( decoder->stop( FrontendDecoder::Exiting ) )
      e->ignore(); // the decoder is running, wait
   else
      e->accept(); // the decoder is not running, exit
}

void KMpg::openClicked()
{
   if (filedialog==NULL)
      filedialog = new QFileDialog( NULL, "*.mp?", this, "Open file", TRUE );
   filedialog->setCaption( klocale->translate("Open file") );

   if( filedialog->exec() == 1)
      playlistMgr->insertPlay( KURL(filedialog->selectedFile()) );
}

void KMpg::stopped(FrontendDecoder::stopReason stopReason)
{
   ledPanel->stop();
   cerr << "KMpg:: stopped: Stop signal received" << endl;
   if( stopReason==FrontendDecoder::Command ) {
      cerr << "KMpg:: stopped: Reason is Command" << endl;
      return;
   }
   else if( stopReason==FrontendDecoder::Exiting )
   {
      cerr << "KMpg:: stopped: Reason is Exiting" << endl;
      KApplication::getKApplication()->quit();
      return;
   }
   else if( stopReason==FrontendDecoder::DecoderCrash )
      QMessageBox::warning( this, 
			    klocale->translate("Decoder crash"),
			    klocale->translate("The decoder has crashed, this is probably " \
					       "due to an error\nin the stream which the " \
					       "decoder is unable to recover from.\n" \
					       "This will be handled more gracefully in later " \
					       "versions of kmpg.") );
   else if( stopReason==FrontendDecoder::Error )
   {
      QMessageBox::critical( this, 
			     klocale->translate("Decoder error"), 
			     klocale->translate("The decoder was unable to open " \
						"the audio device.") );
      return;
   }
   playlistMgr->playNext();
   cerr << "KMpg:: stopped: playing next " << endl;
}

void KMpg::ledPanelResized()
{
   updateRects();
}

void KMpg::openUrl( KURL _url, bool isList )
{
   int retval;
   char *path = _url.path();
   Fileinfo _finfo(_url.path());

   if (!_finfo.isOk())
   {
      QMessageBox::warning( this, 
			    klocale->translate("Error"),
			    klocale->translate("The selected file is not valid\n"\
					       "and cannot be played by this " \
					       "version of KMpg") );
      return;
   }

   decoder->stop(FrontendDecoder::Command);   
   decoder->closeFile();   
   decoder->openFile( path );
   if ( isList )
      playlistMgr->setList(TRUE);
   else {
      playlistMgr->setList(FALSE);
      ledPanel->setField( 0 , "-/-" );
   }
   ledPanel->setInfo( _finfo );

   decoder->play();
}

void KMpg::onDrop( KDNDDropZone* _zone )
{
   QStrList strlist;
   KURL *url;
   char *str;

   strlist = _zone->getURLList();

   // New version, handles dropping of directories too
   playlistMgr->clear(); // Hmm, should we really do this????

   for( str=strlist.first(); str!=0; str=strlist.next() )
   {
      url = new KURL( str );
      playlistMgr->insert( *url );
      delete url;
   }
   decoder->stop(FrontendDecoder::Command);
   playlistMgr->playNext();
}
